@@ -1,4 +1,4 @@
-from .aws import S3Location
+from .aws import S3Location, LoadPartitions
 from .db import CachedAthenaQuery, CachedPep249Query, BaseCachedQuery, AthenaClient, SqlClient
 from .doggo import PandasDoggo, FileDoggo, CSVDoggo, DoggoFileSystem, DoggoLock, DoggoWait, DynamoDogLock
 from .log import log_to_stdout, PersistentFieldLogger
@@ -9,5 +9,5 @@ __all__ = ['S3Location',
            'PandasDoggo', 'FileDoggo', 'CSVDoggo', 'DoggoFileSystem',
            'DoggoLock', 'DoggoWait', 'log_to_stdout', 'PersistentFieldLogger',
            'AthenaClient', 'SqlClient',
-           'BaseTest'
+           'BaseTest', 'LoadPartitions'
            ]
\ No newline at end of file
