@@ -36,13 +36,12 @@ class S3List:
                 for i in page['CommonPrefixes']:
                     result_list.add(i['Prefix'])
 
-
         return result_list
 
 
-class AthenaPartition:
+class LoadPartitions:
     """
-    The AthenaPartition class will recurse to a specified depth in S3 and create SQL to add all partitions
+    The LoadPartitions class will recurse to a specified depth in S3 and create SQL to add all partitions
     to a table in Athena. This is significantly faster than using ``MSCK REPAIR TABLE`` as it only searches
     to the specified depth.
 
@@ -183,3 +182,16 @@ class AthenaPartition:
         partitions_list = self.list_partitions(table, partition_depth, s3_path)
 
         return self.generate_sql(table, partitions_list, s3_path, athena_client, output_location, athena_s3_client)
+
+    def get_sql_from_file_names(self, table, file_names, s3_path=None, athena_client=None, output_location=None,
+                                athena_s3_client=None):
+        """
+        Adds functionality to get the load partitions sql from the output of save_partitioned (PandasDoggo).
+        All other params are the same as in get_sql (with removal of partition_depth as we do not use list_partitions).
+
+        :param file_names: Full file names (as output from PandasDoggo.save_partitioned)
+        """
+
+        partitions_list = [S3Location(file).prefix for file in file_names]
+
+        return self.generate_sql(table, partitions_list, s3_path, athena_client, output_location, athena_s3_client)
