@@ -399,6 +399,124 @@ class TestDoggoFileSystem(BaseTest):
 
         self.assertEqual(dfs.size(file_2), 4)
 
+    def test_ls(self):
+        dfs = DoggoFileSystem()
+        root = self.test_dir
+        testdir = dfs.join(root, str(random()))
+        file_1 = dfs.join(testdir, "temp_1.txt")
+        dir_local_1 = testdir + '/'
+        dir_s3 = S3Location('newtools-testing').join('dfs/')
+
+        with dfs.open(file_1, "wb") as f:
+            f.write(b"woof")
+
+        dfs.cp(dir_local_1, dir_s3)
+
+        contents = dfs.ls(os.path.join(root, 'pandas_doggo_tests/csv'), recursive=True)
+        self.assertEqual(len(contents), 4)
+        self.assertTrue(any('write_target.csv.gzip' in item for item in contents))
+        self.assertTrue(any('test.csv.gz' in item for item in contents))
+        self.assertTrue(any('uncompressed.csv' in item for item in contents))
+        self.assertTrue(any('email_test.csv' in item for item in contents))
+
+        self.assertEqual(len(dfs.ls(dir_local_1)), 1)
+        self.assertTrue('temp_1.txt' in dfs.ls(dir_local_1)[0])
+
+        self.assertEqual(len(dfs.ls('s3://totallyfakelocation')), 0)
+        self.assertEqual(len(dfs.ls('s3://totallyfakelocation', recursive=True)), 0)
+        self.assertEqual(len(dfs.ls(os.path.join(dir_local_1, 'totallyfakefolder'))), 0)
+        self.assertEqual(len(dfs.ls(os.path.join(dir_local_1, 'totallyfakefolder'), recursive=True)), 0)
+
+        self.assertEqual(dfs.ls(dir_s3), [S3Location('s3://newtools-testing/dfs/temp_1.txt')])
+
+        self.assertEqual(dfs.ls(S3Location(dir_s3), recursive=True),
+                         [S3Location('s3://newtools-testing/dfs/temp_1.txt')])
+
+    def test_mixed_local_s3(self):
+        dfs = DoggoFileSystem()
+        root = self.tempdir
+        testdir = dfs.join(root, str(random()))
+
+        file_1 = dfs.join(testdir, "temp_1.txt")
+        file_2 = dfs.join(testdir, "tests", "temp_2.txt")
+        dir_local_1 = testdir + '/'
+        dir_local_2 = dfs.join(root, str(random())) + '/'
+        dir_s3 = S3Location('newtools-testing').join('dfs')
+        file_3 = dfs.join(dir_local_2, "temp_1.txt")
+        file_4 = dfs.join(dir_local_2, "tests", "temp_2.txt")
+        s3_loc_1 = S3Location('newtools-testing').join('dfs', 'temp_1.txt')
+        s3_loc_2 = S3Location('newtools-testing').join('dfs', 'temp_2.txt')
+        dfs.rm(s3_loc_1)
+        dfs.rm(s3_loc_2)
+
+        with self.assertRaises(FileNotFoundError):
+            dfs.cp(s3_loc_1, s3_loc_2)
+
+        with self.assertRaises(FileNotFoundError):
+            dfs.mv(s3_loc_1, s3_loc_2)
+
+        self.assertEqual(dfs.split(file_1), (testdir, "temp_1.txt"))
+
+        self.assertFalse(dfs.exists(file_1))
+        self.assertFalse(dfs.exists(file_2))
+
+        with dfs.open(file_1, "wb") as f:
+            f.write(b"woof")
+
+        self.assertTrue(dfs.exists(file_1))
+        self.assertFalse(dfs.exists(file_2))
+        self.assertFalse(dfs.exists(file_3))
+        self.assertFalse(dfs.exists(file_4))
+
+        dfs.cp(file_1, file_2)
+        self.assertTrue(dfs.exists(file_1))
+        self.assertTrue(dfs.exists(file_2))
+        dfs.cp(dir_local_1, dir_local_2)
+        self.assertTrue(dfs.exists(file_1))
+        self.assertTrue(dfs.exists(file_2))
+        self.assertTrue(dfs.exists(file_3))
+        self.assertTrue(dfs.exists(file_4))
+        shutil.rmtree(dir_local_2)
+        dfs.cp(file_1, s3_loc_1)
+        self.assertTrue(dfs.exists(s3_loc_1))
+        dfs.cp(s3_loc_1, s3_loc_2)
+        self.assertTrue(dfs.exists(s3_loc_2))
+
+        paths = dfs.glob(dfs.join(testdir, "**"))
+        self.assertIn(file_1, paths)
+        self.assertIn(file_2, paths)
+
+        dfs.rm(file_2)
+        self.assertTrue(dfs.exists(file_1))
+        self.assertFalse(dfs.exists(file_2))
+        dfs.cp(s3_loc_2, file_2)
+        self.assertTrue(dfs.exists(file_2))
+        dfs.rm(file_2)
+
+        dfs.mv(file_1, file_2)
+        self.assertFalse(dfs.exists(file_1))
+        self.assertTrue(dfs.exists(file_2))
+        dfs.mv(file_2, s3_loc_1)
+        self.assertFalse(dfs.exists(file_2))
+        self.assertTrue(dfs.exists(s3_loc_1))
+        dfs.mv(s3_loc_1, s3_loc_2)
+        self.assertFalse(dfs.exists(s3_loc_1))
+        self.assertTrue(dfs.exists(s3_loc_2))
+        dfs.mv(s3_loc_2, file_1)
+        self.assertFalse(dfs.exists(s3_loc_2))
+        self.assertTrue(dfs.exists(file_1))
+
+        dfs.mv(dir_local_1, dir_local_2)
+        self.assertFalse(dfs.exists(file_1))
+        self.assertTrue(dfs.exists(file_3))
+
+        dfs.mv(dir_local_2, dir_s3)
+        self.assertFalse(dfs.exists(file_3))
+        self.assertTrue(dfs.exists(s3_loc_1))
+
+        dfs.rm(s3_loc_1)
+        dfs.rm(s3_loc_2)
+
     def test_s3(self):
         self._test(S3Location("s3://07092018pqtest/fs/"))
 
@@ -408,11 +526,6 @@ class TestDoggoFileSystem(BaseTest):
     def test_file(self):
         self._test(self.tempdir)
 
-    def test_bad_copy(self):
-        with self.assertRaises(NotImplementedError):
-            DoggoFileSystem().cp("s3://07092018pqtest/fs/file.tmp",
-                                 os.path.join(self.tempdir, "file.tmp"))
-
 
 class TestDoggoWait(unittest.TestCase):
     def test_basic(self):
